# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.cloudforce_one.threat_events import dataset_edit_params, dataset_create_params
from ....types.cloudforce_one.threat_events.dataset_get_response import DatasetGetResponse
from ....types.cloudforce_one.threat_events.dataset_raw_response import DatasetRawResponse
from ....types.cloudforce_one.threat_events.dataset_edit_response import DatasetEditResponse
from ....types.cloudforce_one.threat_events.dataset_list_response import DatasetListResponse
from ....types.cloudforce_one.threat_events.dataset_create_response import DatasetCreateResponse

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        is_public: bool,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateResponse:
        """
        Creates a dataset

        Args:
          account_id: Account ID.

          is_public: If true, then anyone can search the dataset. If false, then its limited to the
              account.

          name: Used to describe the dataset within the account context.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/cloudforce-one/events/dataset/create", account_id=account_id),
            body=maybe_transform(
                {
                    "is_public": is_public,
                    "name": name,
                },
                dataset_create_params.DatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateResponse,
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListResponse:
        """
        Lists all datasets in an account

        Args:
          account_id: Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/dataset", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListResponse,
        )

    def edit(
        self,
        dataset_id: str,
        *,
        account_id: str,
        is_public: bool,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetEditResponse:
        """
        Updates an existing dataset

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          is_public: If true, then anyone can search the dataset. If false, then its limited to the
              account.

          name: Used to describe the dataset within the account context.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}",
                account_id=account_id,
                dataset_id=dataset_id,
            ),
            body=maybe_transform(
                {
                    "is_public": is_public,
                    "name": name,
                },
                dataset_edit_params.DatasetEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetEditResponse,
        )

    def get(
        self,
        dataset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetGetResponse:
        """
        Reads a dataset

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}",
                account_id=account_id,
                dataset_id=dataset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetResponse,
        )

    def raw(
        self,
        event_id: str,
        *,
        account_id: str,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetRawResponse:
        """Retrieves the raw data associated with an event.

        Searches across all shards in
        the dataset.

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          event_id: Event ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/raw/{dataset_id}/{event_id}",
                account_id=account_id,
                dataset_id=dataset_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetRawResponse,
        )


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        is_public: bool,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateResponse:
        """
        Creates a dataset

        Args:
          account_id: Account ID.

          is_public: If true, then anyone can search the dataset. If false, then its limited to the
              account.

          name: Used to describe the dataset within the account context.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/cloudforce-one/events/dataset/create", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "is_public": is_public,
                    "name": name,
                },
                dataset_create_params.DatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateResponse,
        )

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetListResponse:
        """
        Lists all datasets in an account

        Args:
          account_id: Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/dataset", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListResponse,
        )

    async def edit(
        self,
        dataset_id: str,
        *,
        account_id: str,
        is_public: bool,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetEditResponse:
        """
        Updates an existing dataset

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          is_public: If true, then anyone can search the dataset. If false, then its limited to the
              account.

          name: Used to describe the dataset within the account context.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}",
                account_id=account_id,
                dataset_id=dataset_id,
            ),
            body=await async_maybe_transform(
                {
                    "is_public": is_public,
                    "name": name,
                },
                dataset_edit_params.DatasetEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetEditResponse,
        )

    async def get(
        self,
        dataset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetGetResponse:
        """
        Reads a dataset

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}",
                account_id=account_id,
                dataset_id=dataset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetResponse,
        )

    async def raw(
        self,
        event_id: str,
        *,
        account_id: str,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetRawResponse:
        """Retrieves the raw data associated with an event.

        Searches across all shards in
        the dataset.

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          event_id: Event ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/raw/{dataset_id}/{event_id}",
                account_id=account_id,
                dataset_id=dataset_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetRawResponse,
        )


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.create = to_raw_response_wrapper(
            datasets.create,
        )
        self.list = to_raw_response_wrapper(
            datasets.list,
        )
        self.edit = to_raw_response_wrapper(
            datasets.edit,
        )
        self.get = to_raw_response_wrapper(
            datasets.get,
        )
        self.raw = to_raw_response_wrapper(
            datasets.raw,
        )


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.create = async_to_raw_response_wrapper(
            datasets.create,
        )
        self.list = async_to_raw_response_wrapper(
            datasets.list,
        )
        self.edit = async_to_raw_response_wrapper(
            datasets.edit,
        )
        self.get = async_to_raw_response_wrapper(
            datasets.get,
        )
        self.raw = async_to_raw_response_wrapper(
            datasets.raw,
        )


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.create = to_streamed_response_wrapper(
            datasets.create,
        )
        self.list = to_streamed_response_wrapper(
            datasets.list,
        )
        self.edit = to_streamed_response_wrapper(
            datasets.edit,
        )
        self.get = to_streamed_response_wrapper(
            datasets.get,
        )
        self.raw = to_streamed_response_wrapper(
            datasets.raw,
        )


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.create = async_to_streamed_response_wrapper(
            datasets.create,
        )
        self.list = async_to_streamed_response_wrapper(
            datasets.list,
        )
        self.edit = async_to_streamed_response_wrapper(
            datasets.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            datasets.get,
        )
        self.raw = async_to_streamed_response_wrapper(
            datasets.raw,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["HotlinkProtection"]


class HotlinkProtection(BaseModel):
    """
    When enabled, the Hotlink Protection option ensures that other sites cannot suck up your bandwidth by building pages that use images hosted on your site. Anytime a request for an image on your site hits Cloudflare, we check to ensure that it's not another site requesting them. People will still be able to download and view images from your page, but other sites won't be able to steal them for use on their own pages. (https://support.cloudflare.com/hc/en-us/articles/200170026).
    """

    id: Literal["hotlink_protection"]
    """ID of the zone setting."""

    value: Literal["on", "off"]
    """Current value of the zone setting."""

    editable: Optional[Literal[True, False]] = None
    """
    Whether or not this setting can be modified for this zone (based on your
    Cloudflare plan level).
    """

    modified_on: Optional[datetime] = None
    """last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, Iterable, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .authorities import (
    AuthoritiesResource,
    AsyncAuthoritiesResource,
    AuthoritiesResourceWithRawResponse,
    AsyncAuthoritiesResourceWithRawResponse,
    AuthoritiesResourceWithStreamingResponse,
    AsyncAuthoritiesResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....types.radar import ct_summary_params, ct_timeseries_params, ct_timeseries_groups_params
from ...._base_client import make_request_options
from ....types.radar.ct_summary_response import CTSummaryResponse
from ....types.radar.ct_timeseries_response import CTTimeseriesResponse
from ....types.radar.ct_timeseries_groups_response import CTTimeseriesGroupsResponse

__all__ = ["CTResource", "AsyncCTResource"]


class CTResource(SyncAPIResource):
    @cached_property
    def authorities(self) -> AuthoritiesResource:
        return AuthoritiesResource(self._client)

    @cached_property
    def logs(self) -> LogsResource:
        return LogsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CTResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CTResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CTResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CTResourceWithStreamingResponse(self)

    def summary(
        self,
        dimension: Literal[
            "CA",
            "CA_OWNER",
            "DURATION",
            "ENTRY_TYPE",
            "EXPIRATION_STATUS",
            "HAS_IPS",
            "HAS_WILDCARDS",
            "LOG",
            "LOG_API",
            "LOG_OPERATOR",
            "PUBLIC_KEY_ALGORITHM",
            "SIGNATURE_ALGORITHM",
            "TLD",
            "VALIDATION_LEVEL",
        ],
        *,
        ca: SequenceNotStr[str] | Omit = omit,
        ca_owner: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        duration: List[
            Literal[
                "LTE_3D",
                "GT_3D_LTE_7D",
                "GT_7D_LTE_10D",
                "GT_10D_LTE_47D",
                "GT_47D_LTE_100D",
                "GT_100D_LTE_200D",
                "GT_200D",
            ]
        ]
        | Omit = omit,
        entry_type: List[Literal["PRECERTIFICATE", "CERTIFICATE"]] | Omit = omit,
        expiration_status: List[Literal["EXPIRED", "VALID"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        has_ips: Iterable[bool] | Omit = omit,
        has_wildcards: Iterable[bool] | Omit = omit,
        limit_per_group: int | Omit = omit,
        log: SequenceNotStr[str] | Omit = omit,
        log_api: List[Literal["RFC6962", "STATIC"]] | Omit = omit,
        log_operator: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["RAW_VALUES", "PERCENTAGE"] | Omit = omit,
        public_key_algorithm: List[Literal["DSA", "ECDSA", "RSA"]] | Omit = omit,
        signature_algorithm: List[
            Literal[
                "DSA_SHA_1",
                "DSA_SHA_256",
                "ECDSA_SHA_1",
                "ECDSA_SHA_256",
                "ECDSA_SHA_384",
                "ECDSA_SHA_512",
                "PSS_SHA_256",
                "PSS_SHA_384",
                "PSS_SHA_512",
                "RSA_MD2",
                "RSA_MD5",
                "RSA_SHA_1",
                "RSA_SHA_256",
                "RSA_SHA_384",
                "RSA_SHA_512",
            ]
        ]
        | Omit = omit,
        tld: SequenceNotStr[str] | Omit = omit,
        unique_entries: List[Literal["true", "false"]] | Omit = omit,
        validation_level: List[Literal["DOMAIN", "ORGANIZATION", "EXTENDED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CTSummaryResponse:
        """
        Retrieves an aggregated summary of certificates grouped by the specified
        dimension.

        Args:
          dimension: Specifies the certificate attribute by which to group the results.

          ca: Filters results by certificate authority.

          ca_owner: Filters results by certificate authority owner.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          duration: Filters results by certificate duration.

          entry_type: Filters results by entry type (certificate vs. pre-certificate).

          expiration_status: Filters results by expiration status (expired vs. valid).

          format: Format in which results will be returned.

          has_ips: Filters results based on whether the certificates are bound to specific IP
              addresses.

          has_wildcards: Filters results based on whether the certificates contain wildcard domains.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          log: Filters results by certificate log.

          log_api: Filters results by certificate log API (RFC6962 vs. static).

          log_operator: Filters results by certificate log operator.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          public_key_algorithm: Filters results by public key algorithm.

          signature_algorithm: Filters results by signature algorithm.

          tld: Filters results by top-level domain.

          unique_entries: Specifies whether to filter out duplicate certificates and pre-certificates. Set
              to true for unique entries only.

          validation_level: Filters results by validation level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/ct/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ca": ca,
                        "ca_owner": ca_owner,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "duration": duration,
                        "entry_type": entry_type,
                        "expiration_status": expiration_status,
                        "format": format,
                        "has_ips": has_ips,
                        "has_wildcards": has_wildcards,
                        "limit_per_group": limit_per_group,
                        "log": log,
                        "log_api": log_api,
                        "log_operator": log_operator,
                        "name": name,
                        "normalization": normalization,
                        "public_key_algorithm": public_key_algorithm,
                        "signature_algorithm": signature_algorithm,
                        "tld": tld,
                        "unique_entries": unique_entries,
                        "validation_level": validation_level,
                    },
                    ct_summary_params.CTSummaryParams,
                ),
                post_parser=ResultWrapper[CTSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[CTSummaryResponse], ResultWrapper[CTSummaryResponse]),
        )

    def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        ca: SequenceNotStr[str] | Omit = omit,
        ca_owner: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        duration: List[
            Literal[
                "LTE_3D",
                "GT_3D_LTE_7D",
                "GT_7D_LTE_10D",
                "GT_10D_LTE_47D",
                "GT_47D_LTE_100D",
                "GT_100D_LTE_200D",
                "GT_200D",
            ]
        ]
        | Omit = omit,
        entry_type: List[Literal["PRECERTIFICATE", "CERTIFICATE"]] | Omit = omit,
        expiration_status: List[Literal["EXPIRED", "VALID"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        has_ips: Iterable[bool] | Omit = omit,
        has_wildcards: Iterable[bool] | Omit = omit,
        log: SequenceNotStr[str] | Omit = omit,
        log_api: List[Literal["RFC6962", "STATIC"]] | Omit = omit,
        log_operator: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        public_key_algorithm: List[Literal["DSA", "ECDSA", "RSA"]] | Omit = omit,
        signature_algorithm: List[
            Literal[
                "DSA_SHA_1",
                "DSA_SHA_256",
                "ECDSA_SHA_1",
                "ECDSA_SHA_256",
                "ECDSA_SHA_384",
                "ECDSA_SHA_512",
                "PSS_SHA_256",
                "PSS_SHA_384",
                "PSS_SHA_512",
                "RSA_MD2",
                "RSA_MD5",
                "RSA_SHA_1",
                "RSA_SHA_256",
                "RSA_SHA_384",
                "RSA_SHA_512",
            ]
        ]
        | Omit = omit,
        tld: SequenceNotStr[str] | Omit = omit,
        unique_entries: List[Literal["true", "false"]] | Omit = omit,
        validation_level: List[Literal["DOMAIN", "ORGANIZATION", "EXTENDED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CTTimeseriesResponse:
        """
        Retrieves certificate volume over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          ca: Filters results by certificate authority.

          ca_owner: Filters results by certificate authority owner.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          duration: Filters results by certificate duration.

          entry_type: Filters results by entry type (certificate vs. pre-certificate).

          expiration_status: Filters results by expiration status (expired vs. valid).

          format: Format in which results will be returned.

          has_ips: Filters results based on whether the certificates are bound to specific IP
              addresses.

          has_wildcards: Filters results based on whether the certificates contain wildcard domains.

          log: Filters results by certificate log.

          log_api: Filters results by certificate log API (RFC6962 vs. static).

          log_operator: Filters results by certificate log operator.

          name: Array of names used to label the series in the response.

          public_key_algorithm: Filters results by public key algorithm.

          signature_algorithm: Filters results by signature algorithm.

          tld: Filters results by top-level domain.

          unique_entries: Specifies whether to filter out duplicate certificates and pre-certificates. Set
              to true for unique entries only.

          validation_level: Filters results by validation level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ct/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "ca": ca,
                        "ca_owner": ca_owner,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "duration": duration,
                        "entry_type": entry_type,
                        "expiration_status": expiration_status,
                        "format": format,
                        "has_ips": has_ips,
                        "has_wildcards": has_wildcards,
                        "log": log,
                        "log_api": log_api,
                        "log_operator": log_operator,
                        "name": name,
                        "public_key_algorithm": public_key_algorithm,
                        "signature_algorithm": signature_algorithm,
                        "tld": tld,
                        "unique_entries": unique_entries,
                        "validation_level": validation_level,
                    },
                    ct_timeseries_params.CTTimeseriesParams,
                ),
                post_parser=ResultWrapper[CTTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[CTTimeseriesResponse], ResultWrapper[CTTimeseriesResponse]),
        )

    def timeseries_groups(
        self,
        dimension: Literal[
            "CA",
            "CA_OWNER",
            "DURATION",
            "ENTRY_TYPE",
            "EXPIRATION_STATUS",
            "HAS_IPS",
            "HAS_WILDCARDS",
            "LOG",
            "LOG_API",
            "LOG_OPERATOR",
            "PUBLIC_KEY_ALGORITHM",
            "SIGNATURE_ALGORITHM",
            "TLD",
            "VALIDATION_LEVEL",
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        ca: SequenceNotStr[str] | Omit = omit,
        ca_owner: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        duration: List[
            Literal[
                "LTE_3D",
                "GT_3D_LTE_7D",
                "GT_7D_LTE_10D",
                "GT_10D_LTE_47D",
                "GT_47D_LTE_100D",
                "GT_100D_LTE_200D",
                "GT_200D",
            ]
        ]
        | Omit = omit,
        entry_type: List[Literal["PRECERTIFICATE", "CERTIFICATE"]] | Omit = omit,
        expiration_status: List[Literal["EXPIRED", "VALID"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        has_ips: Iterable[bool] | Omit = omit,
        has_wildcards: Iterable[bool] | Omit = omit,
        limit_per_group: int | Omit = omit,
        log: SequenceNotStr[str] | Omit = omit,
        log_api: List[Literal["RFC6962", "STATIC"]] | Omit = omit,
        log_operator: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["RAW_VALUES", "PERCENTAGE"] | Omit = omit,
        public_key_algorithm: List[Literal["DSA", "ECDSA", "RSA"]] | Omit = omit,
        signature_algorithm: List[
            Literal[
                "DSA_SHA_1",
                "DSA_SHA_256",
                "ECDSA_SHA_1",
                "ECDSA_SHA_256",
                "ECDSA_SHA_384",
                "ECDSA_SHA_512",
                "PSS_SHA_256",
                "PSS_SHA_384",
                "PSS_SHA_512",
                "RSA_MD2",
                "RSA_MD5",
                "RSA_SHA_1",
                "RSA_SHA_256",
                "RSA_SHA_384",
                "RSA_SHA_512",
            ]
        ]
        | Omit = omit,
        tld: SequenceNotStr[str] | Omit = omit,
        unique_entries: List[Literal["true", "false"]] | Omit = omit,
        validation_level: List[Literal["DOMAIN", "ORGANIZATION", "EXTENDED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CTTimeseriesGroupsResponse:
        """
        Retrieves the distribution of certificates grouped by the specified dimension
        over time.

        Args:
          dimension: Specifies the certificate attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          ca: Filters results by certificate authority.

          ca_owner: Filters results by certificate authority owner.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          duration: Filters results by certificate duration.

          entry_type: Filters results by entry type (certificate vs. pre-certificate).

          expiration_status: Filters results by expiration status (expired vs. valid).

          format: Format in which results will be returned.

          has_ips: Filters results based on whether the certificates are bound to specific IP
              addresses.

          has_wildcards: Filters results based on whether the certificates contain wildcard domains.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          log: Filters results by certificate log.

          log_api: Filters results by certificate log API (RFC6962 vs. static).

          log_operator: Filters results by certificate log operator.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          public_key_algorithm: Filters results by public key algorithm.

          signature_algorithm: Filters results by signature algorithm.

          tld: Filters results by top-level domain.

          unique_entries: Specifies whether to filter out duplicate certificates and pre-certificates. Set
              to true for unique entries only.

          validation_level: Filters results by validation level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/ct/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "ca": ca,
                        "ca_owner": ca_owner,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "duration": duration,
                        "entry_type": entry_type,
                        "expiration_status": expiration_status,
                        "format": format,
                        "has_ips": has_ips,
                        "has_wildcards": has_wildcards,
                        "limit_per_group": limit_per_group,
                        "log": log,
                        "log_api": log_api,
                        "log_operator": log_operator,
                        "name": name,
                        "normalization": normalization,
                        "public_key_algorithm": public_key_algorithm,
                        "signature_algorithm": signature_algorithm,
                        "tld": tld,
                        "unique_entries": unique_entries,
                        "validation_level": validation_level,
                    },
                    ct_timeseries_groups_params.CTTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[CTTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[CTTimeseriesGroupsResponse], ResultWrapper[CTTimeseriesGroupsResponse]),
        )


class AsyncCTResource(AsyncAPIResource):
    @cached_property
    def authorities(self) -> AsyncAuthoritiesResource:
        return AsyncAuthoritiesResource(self._client)

    @cached_property
    def logs(self) -> AsyncLogsResource:
        return AsyncLogsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCTResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCTResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCTResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCTResourceWithStreamingResponse(self)

    async def summary(
        self,
        dimension: Literal[
            "CA",
            "CA_OWNER",
            "DURATION",
            "ENTRY_TYPE",
            "EXPIRATION_STATUS",
            "HAS_IPS",
            "HAS_WILDCARDS",
            "LOG",
            "LOG_API",
            "LOG_OPERATOR",
            "PUBLIC_KEY_ALGORITHM",
            "SIGNATURE_ALGORITHM",
            "TLD",
            "VALIDATION_LEVEL",
        ],
        *,
        ca: SequenceNotStr[str] | Omit = omit,
        ca_owner: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        duration: List[
            Literal[
                "LTE_3D",
                "GT_3D_LTE_7D",
                "GT_7D_LTE_10D",
                "GT_10D_LTE_47D",
                "GT_47D_LTE_100D",
                "GT_100D_LTE_200D",
                "GT_200D",
            ]
        ]
        | Omit = omit,
        entry_type: List[Literal["PRECERTIFICATE", "CERTIFICATE"]] | Omit = omit,
        expiration_status: List[Literal["EXPIRED", "VALID"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        has_ips: Iterable[bool] | Omit = omit,
        has_wildcards: Iterable[bool] | Omit = omit,
        limit_per_group: int | Omit = omit,
        log: SequenceNotStr[str] | Omit = omit,
        log_api: List[Literal["RFC6962", "STATIC"]] | Omit = omit,
        log_operator: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["RAW_VALUES", "PERCENTAGE"] | Omit = omit,
        public_key_algorithm: List[Literal["DSA", "ECDSA", "RSA"]] | Omit = omit,
        signature_algorithm: List[
            Literal[
                "DSA_SHA_1",
                "DSA_SHA_256",
                "ECDSA_SHA_1",
                "ECDSA_SHA_256",
                "ECDSA_SHA_384",
                "ECDSA_SHA_512",
                "PSS_SHA_256",
                "PSS_SHA_384",
                "PSS_SHA_512",
                "RSA_MD2",
                "RSA_MD5",
                "RSA_SHA_1",
                "RSA_SHA_256",
                "RSA_SHA_384",
                "RSA_SHA_512",
            ]
        ]
        | Omit = omit,
        tld: SequenceNotStr[str] | Omit = omit,
        unique_entries: List[Literal["true", "false"]] | Omit = omit,
        validation_level: List[Literal["DOMAIN", "ORGANIZATION", "EXTENDED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CTSummaryResponse:
        """
        Retrieves an aggregated summary of certificates grouped by the specified
        dimension.

        Args:
          dimension: Specifies the certificate attribute by which to group the results.

          ca: Filters results by certificate authority.

          ca_owner: Filters results by certificate authority owner.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          duration: Filters results by certificate duration.

          entry_type: Filters results by entry type (certificate vs. pre-certificate).

          expiration_status: Filters results by expiration status (expired vs. valid).

          format: Format in which results will be returned.

          has_ips: Filters results based on whether the certificates are bound to specific IP
              addresses.

          has_wildcards: Filters results based on whether the certificates contain wildcard domains.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          log: Filters results by certificate log.

          log_api: Filters results by certificate log API (RFC6962 vs. static).

          log_operator: Filters results by certificate log operator.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          public_key_algorithm: Filters results by public key algorithm.

          signature_algorithm: Filters results by signature algorithm.

          tld: Filters results by top-level domain.

          unique_entries: Specifies whether to filter out duplicate certificates and pre-certificates. Set
              to true for unique entries only.

          validation_level: Filters results by validation level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/ct/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ca": ca,
                        "ca_owner": ca_owner,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "duration": duration,
                        "entry_type": entry_type,
                        "expiration_status": expiration_status,
                        "format": format,
                        "has_ips": has_ips,
                        "has_wildcards": has_wildcards,
                        "limit_per_group": limit_per_group,
                        "log": log,
                        "log_api": log_api,
                        "log_operator": log_operator,
                        "name": name,
                        "normalization": normalization,
                        "public_key_algorithm": public_key_algorithm,
                        "signature_algorithm": signature_algorithm,
                        "tld": tld,
                        "unique_entries": unique_entries,
                        "validation_level": validation_level,
                    },
                    ct_summary_params.CTSummaryParams,
                ),
                post_parser=ResultWrapper[CTSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[CTSummaryResponse], ResultWrapper[CTSummaryResponse]),
        )

    async def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        ca: SequenceNotStr[str] | Omit = omit,
        ca_owner: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        duration: List[
            Literal[
                "LTE_3D",
                "GT_3D_LTE_7D",
                "GT_7D_LTE_10D",
                "GT_10D_LTE_47D",
                "GT_47D_LTE_100D",
                "GT_100D_LTE_200D",
                "GT_200D",
            ]
        ]
        | Omit = omit,
        entry_type: List[Literal["PRECERTIFICATE", "CERTIFICATE"]] | Omit = omit,
        expiration_status: List[Literal["EXPIRED", "VALID"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        has_ips: Iterable[bool] | Omit = omit,
        has_wildcards: Iterable[bool] | Omit = omit,
        log: SequenceNotStr[str] | Omit = omit,
        log_api: List[Literal["RFC6962", "STATIC"]] | Omit = omit,
        log_operator: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        public_key_algorithm: List[Literal["DSA", "ECDSA", "RSA"]] | Omit = omit,
        signature_algorithm: List[
            Literal[
                "DSA_SHA_1",
                "DSA_SHA_256",
                "ECDSA_SHA_1",
                "ECDSA_SHA_256",
                "ECDSA_SHA_384",
                "ECDSA_SHA_512",
                "PSS_SHA_256",
                "PSS_SHA_384",
                "PSS_SHA_512",
                "RSA_MD2",
                "RSA_MD5",
                "RSA_SHA_1",
                "RSA_SHA_256",
                "RSA_SHA_384",
                "RSA_SHA_512",
            ]
        ]
        | Omit = omit,
        tld: SequenceNotStr[str] | Omit = omit,
        unique_entries: List[Literal["true", "false"]] | Omit = omit,
        validation_level: List[Literal["DOMAIN", "ORGANIZATION", "EXTENDED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CTTimeseriesResponse:
        """
        Retrieves certificate volume over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          ca: Filters results by certificate authority.

          ca_owner: Filters results by certificate authority owner.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          duration: Filters results by certificate duration.

          entry_type: Filters results by entry type (certificate vs. pre-certificate).

          expiration_status: Filters results by expiration status (expired vs. valid).

          format: Format in which results will be returned.

          has_ips: Filters results based on whether the certificates are bound to specific IP
              addresses.

          has_wildcards: Filters results based on whether the certificates contain wildcard domains.

          log: Filters results by certificate log.

          log_api: Filters results by certificate log API (RFC6962 vs. static).

          log_operator: Filters results by certificate log operator.

          name: Array of names used to label the series in the response.

          public_key_algorithm: Filters results by public key algorithm.

          signature_algorithm: Filters results by signature algorithm.

          tld: Filters results by top-level domain.

          unique_entries: Specifies whether to filter out duplicate certificates and pre-certificates. Set
              to true for unique entries only.

          validation_level: Filters results by validation level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ct/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "ca": ca,
                        "ca_owner": ca_owner,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "duration": duration,
                        "entry_type": entry_type,
                        "expiration_status": expiration_status,
                        "format": format,
                        "has_ips": has_ips,
                        "has_wildcards": has_wildcards,
                        "log": log,
                        "log_api": log_api,
                        "log_operator": log_operator,
                        "name": name,
                        "public_key_algorithm": public_key_algorithm,
                        "signature_algorithm": signature_algorithm,
                        "tld": tld,
                        "unique_entries": unique_entries,
                        "validation_level": validation_level,
                    },
                    ct_timeseries_params.CTTimeseriesParams,
                ),
                post_parser=ResultWrapper[CTTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[CTTimeseriesResponse], ResultWrapper[CTTimeseriesResponse]),
        )

    async def timeseries_groups(
        self,
        dimension: Literal[
            "CA",
            "CA_OWNER",
            "DURATION",
            "ENTRY_TYPE",
            "EXPIRATION_STATUS",
            "HAS_IPS",
            "HAS_WILDCARDS",
            "LOG",
            "LOG_API",
            "LOG_OPERATOR",
            "PUBLIC_KEY_ALGORITHM",
            "SIGNATURE_ALGORITHM",
            "TLD",
            "VALIDATION_LEVEL",
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        ca: SequenceNotStr[str] | Omit = omit,
        ca_owner: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        duration: List[
            Literal[
                "LTE_3D",
                "GT_3D_LTE_7D",
                "GT_7D_LTE_10D",
                "GT_10D_LTE_47D",
                "GT_47D_LTE_100D",
                "GT_100D_LTE_200D",
                "GT_200D",
            ]
        ]
        | Omit = omit,
        entry_type: List[Literal["PRECERTIFICATE", "CERTIFICATE"]] | Omit = omit,
        expiration_status: List[Literal["EXPIRED", "VALID"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        has_ips: Iterable[bool] | Omit = omit,
        has_wildcards: Iterable[bool] | Omit = omit,
        limit_per_group: int | Omit = omit,
        log: SequenceNotStr[str] | Omit = omit,
        log_api: List[Literal["RFC6962", "STATIC"]] | Omit = omit,
        log_operator: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["RAW_VALUES", "PERCENTAGE"] | Omit = omit,
        public_key_algorithm: List[Literal["DSA", "ECDSA", "RSA"]] | Omit = omit,
        signature_algorithm: List[
            Literal[
                "DSA_SHA_1",
                "DSA_SHA_256",
                "ECDSA_SHA_1",
                "ECDSA_SHA_256",
                "ECDSA_SHA_384",
                "ECDSA_SHA_512",
                "PSS_SHA_256",
                "PSS_SHA_384",
                "PSS_SHA_512",
                "RSA_MD2",
                "RSA_MD5",
                "RSA_SHA_1",
                "RSA_SHA_256",
                "RSA_SHA_384",
                "RSA_SHA_512",
            ]
        ]
        | Omit = omit,
        tld: SequenceNotStr[str] | Omit = omit,
        unique_entries: List[Literal["true", "false"]] | Omit = omit,
        validation_level: List[Literal["DOMAIN", "ORGANIZATION", "EXTENDED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CTTimeseriesGroupsResponse:
        """
        Retrieves the distribution of certificates grouped by the specified dimension
        over time.

        Args:
          dimension: Specifies the certificate attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          ca: Filters results by certificate authority.

          ca_owner: Filters results by certificate authority owner.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          duration: Filters results by certificate duration.

          entry_type: Filters results by entry type (certificate vs. pre-certificate).

          expiration_status: Filters results by expiration status (expired vs. valid).

          format: Format in which results will be returned.

          has_ips: Filters results based on whether the certificates are bound to specific IP
              addresses.

          has_wildcards: Filters results based on whether the certificates contain wildcard domains.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          log: Filters results by certificate log.

          log_api: Filters results by certificate log API (RFC6962 vs. static).

          log_operator: Filters results by certificate log operator.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          public_key_algorithm: Filters results by public key algorithm.

          signature_algorithm: Filters results by signature algorithm.

          tld: Filters results by top-level domain.

          unique_entries: Specifies whether to filter out duplicate certificates and pre-certificates. Set
              to true for unique entries only.

          validation_level: Filters results by validation level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/ct/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "ca": ca,
                        "ca_owner": ca_owner,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "duration": duration,
                        "entry_type": entry_type,
                        "expiration_status": expiration_status,
                        "format": format,
                        "has_ips": has_ips,
                        "has_wildcards": has_wildcards,
                        "limit_per_group": limit_per_group,
                        "log": log,
                        "log_api": log_api,
                        "log_operator": log_operator,
                        "name": name,
                        "normalization": normalization,
                        "public_key_algorithm": public_key_algorithm,
                        "signature_algorithm": signature_algorithm,
                        "tld": tld,
                        "unique_entries": unique_entries,
                        "validation_level": validation_level,
                    },
                    ct_timeseries_groups_params.CTTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[CTTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[CTTimeseriesGroupsResponse], ResultWrapper[CTTimeseriesGroupsResponse]),
        )


class CTResourceWithRawResponse:
    def __init__(self, ct: CTResource) -> None:
        self._ct = ct

        self.summary = to_raw_response_wrapper(
            ct.summary,
        )
        self.timeseries = to_raw_response_wrapper(
            ct.timeseries,
        )
        self.timeseries_groups = to_raw_response_wrapper(
            ct.timeseries_groups,
        )

    @cached_property
    def authorities(self) -> AuthoritiesResourceWithRawResponse:
        return AuthoritiesResourceWithRawResponse(self._ct.authorities)

    @cached_property
    def logs(self) -> LogsResourceWithRawResponse:
        return LogsResourceWithRawResponse(self._ct.logs)


class AsyncCTResourceWithRawResponse:
    def __init__(self, ct: AsyncCTResource) -> None:
        self._ct = ct

        self.summary = async_to_raw_response_wrapper(
            ct.summary,
        )
        self.timeseries = async_to_raw_response_wrapper(
            ct.timeseries,
        )
        self.timeseries_groups = async_to_raw_response_wrapper(
            ct.timeseries_groups,
        )

    @cached_property
    def authorities(self) -> AsyncAuthoritiesResourceWithRawResponse:
        return AsyncAuthoritiesResourceWithRawResponse(self._ct.authorities)

    @cached_property
    def logs(self) -> AsyncLogsResourceWithRawResponse:
        return AsyncLogsResourceWithRawResponse(self._ct.logs)


class CTResourceWithStreamingResponse:
    def __init__(self, ct: CTResource) -> None:
        self._ct = ct

        self.summary = to_streamed_response_wrapper(
            ct.summary,
        )
        self.timeseries = to_streamed_response_wrapper(
            ct.timeseries,
        )
        self.timeseries_groups = to_streamed_response_wrapper(
            ct.timeseries_groups,
        )

    @cached_property
    def authorities(self) -> AuthoritiesResourceWithStreamingResponse:
        return AuthoritiesResourceWithStreamingResponse(self._ct.authorities)

    @cached_property
    def logs(self) -> LogsResourceWithStreamingResponse:
        return LogsResourceWithStreamingResponse(self._ct.logs)


class AsyncCTResourceWithStreamingResponse:
    def __init__(self, ct: AsyncCTResource) -> None:
        self._ct = ct

        self.summary = async_to_streamed_response_wrapper(
            ct.summary,
        )
        self.timeseries = async_to_streamed_response_wrapper(
            ct.timeseries,
        )
        self.timeseries_groups = async_to_streamed_response_wrapper(
            ct.timeseries_groups,
        )

    @cached_property
    def authorities(self) -> AsyncAuthoritiesResourceWithStreamingResponse:
        return AsyncAuthoritiesResourceWithStreamingResponse(self._ct.authorities)

    @cached_property
    def logs(self) -> AsyncLogsResourceWithStreamingResponse:
        return AsyncLogsResourceWithStreamingResponse(self._ct.logs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel
from .saas_app_name_id_format import SaaSAppNameIDFormat

__all__ = ["SAMLSaaSApp", "CustomAttribute", "CustomAttributeSource", "CustomAttributeSourceNameByIdP"]


class CustomAttributeSourceNameByIdP(BaseModel):
    idp_id: Optional[str] = None
    """The UID of the IdP."""

    source_name: Optional[str] = None
    """The name of the IdP provided attribute."""


class CustomAttributeSource(BaseModel):
    name: Optional[str] = None
    """The name of the IdP attribute."""

    name_by_idp: Optional[List[CustomAttributeSourceNameByIdP]] = None
    """A mapping from IdP ID to attribute name."""


class CustomAttribute(BaseModel):
    friendly_name: Optional[str] = None
    """The SAML FriendlyName of the attribute."""

    name: Optional[str] = None
    """The name of the attribute."""

    name_format: Optional[
        Literal[
            "urn:oasis:names:tc:SAML:2.0:attrname-format:unspecified",
            "urn:oasis:names:tc:SAML:2.0:attrname-format:basic",
            "urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
        ]
    ] = None
    """A globally unique name for an identity or service provider."""

    required: Optional[bool] = None
    """If the attribute is required when building a SAML assertion."""

    source: Optional[CustomAttributeSource] = None


class SAMLSaaSApp(BaseModel):
    auth_type: Optional[Literal["saml", "oidc"]] = None
    """Optional identifier indicating the authentication protocol used for the saas
    app.

    Required for OIDC. Default if unset is "saml"
    """

    consumer_service_url: Optional[str] = None
    """
    The service provider's endpoint that is responsible for receiving and parsing a
    SAML assertion.
    """

    custom_attributes: Optional[List[CustomAttribute]] = None

    default_relay_state: Optional[str] = None
    """
    The URL that the user will be redirected to after a successful login for IDP
    initiated logins.
    """

    idp_entity_id: Optional[str] = None
    """The unique identifier for your SaaS application."""

    name_id_format: Optional[SaaSAppNameIDFormat] = None
    """The format of the name identifier sent to the SaaS application."""

    name_id_transform_jsonata: Optional[str] = None
    """
    A [JSONata](https://jsonata.org/) expression that transforms an application's
    user identities into a NameID value for its SAML assertion. This expression
    should evaluate to a singular string. The output of this expression can override
    the `name_id_format` setting.
    """

    public_key: Optional[str] = None
    """The Access public certificate that will be used to verify your identity."""

    saml_attribute_transform_jsonata: Optional[str] = None
    """
    A [JSONata] (https://jsonata.org/) expression that transforms an application's
    user identities into attribute assertions in the SAML response. The expression
    can transform id, email, name, and groups values. It can also transform fields
    listed in the saml_attributes or oidc_fields of the identity provider used to
    authenticate. The output of this expression must be a JSON object.
    """

    sp_entity_id: Optional[str] = None
    """A globally unique name for an identity or service provider."""

    sso_endpoint: Optional[str] = None
    """The endpoint where your SaaS application will send login requests."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.magic_transit import app_edit_params, app_create_params, app_update_params
from ...types.magic_transit.app_edit_response import AppEditResponse
from ...types.magic_transit.app_list_response import AppListResponse
from ...types.magic_transit.app_create_response import AppCreateResponse
from ...types.magic_transit.app_delete_response import AppDeleteResponse
from ...types.magic_transit.app_update_response import AppUpdateResponse

__all__ = ["AppsResource", "AsyncAppsResource"]


class AppsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AppsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AppsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AppsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AppsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        type: str,
        hostnames: SequenceNotStr[str] | Omit = omit,
        ip_subnets: SequenceNotStr[str] | Omit = omit,
        source_subnets: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        """
        Creates a new App for an account

        Args:
          account_id: Identifier

          name: Display name for the app.

          type: Category of the app.

          hostnames: FQDNs to associate with traffic decisions.

          ip_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          source_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/magic/apps", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "hostnames": hostnames,
                    "ip_subnets": ip_subnets,
                    "source_subnets": source_subnets,
                },
                app_create_params.AppCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppCreateResponse]], ResultWrapper[AppCreateResponse]),
        )

    def update(
        self,
        account_app_id: str,
        *,
        account_id: str,
        hostnames: SequenceNotStr[str] | Omit = omit,
        ip_subnets: SequenceNotStr[str] | Omit = omit,
        name: str | Omit = omit,
        source_subnets: SequenceNotStr[str] | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        """
        Updates an Account App

        Args:
          account_id: Identifier

          account_app_id: Identifier

          hostnames: FQDNs to associate with traffic decisions.

          ip_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          name: Display name for the app.

          source_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          type: Category of the app.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not account_app_id:
            raise ValueError(f"Expected a non-empty value for `account_app_id` but received {account_app_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/magic/apps/{account_app_id}",
                account_id=account_id,
                account_app_id=account_app_id,
            ),
            body=maybe_transform(
                {
                    "hostnames": hostnames,
                    "ip_subnets": ip_subnets,
                    "name": name,
                    "source_subnets": source_subnets,
                    "type": type,
                },
                app_update_params.AppUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppUpdateResponse]], ResultWrapper[AppUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[AppListResponse]:
        """
        Lists Apps associated with an account.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/apps", account_id=account_id),
            page=SyncSinglePage[AppListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=cast(Any, AppListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        account_app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppDeleteResponse]:
        """
        Deletes specific Account App.

        Args:
          account_id: Identifier

          account_app_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not account_app_id:
            raise ValueError(f"Expected a non-empty value for `account_app_id` but received {account_app_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/apps/{account_app_id}",
                account_id=account_id,
                account_app_id=account_app_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppDeleteResponse]], ResultWrapper[AppDeleteResponse]),
        )

    def edit(
        self,
        account_app_id: str,
        *,
        account_id: str,
        hostnames: SequenceNotStr[str] | Omit = omit,
        ip_subnets: SequenceNotStr[str] | Omit = omit,
        name: str | Omit = omit,
        source_subnets: SequenceNotStr[str] | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppEditResponse]:
        """
        Updates an Account App

        Args:
          account_id: Identifier

          account_app_id: Identifier

          hostnames: FQDNs to associate with traffic decisions.

          ip_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          name: Display name for the app.

          source_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          type: Category of the app.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not account_app_id:
            raise ValueError(f"Expected a non-empty value for `account_app_id` but received {account_app_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/apps/{account_app_id}",
                account_id=account_id,
                account_app_id=account_app_id,
            ),
            body=maybe_transform(
                {
                    "hostnames": hostnames,
                    "ip_subnets": ip_subnets,
                    "name": name,
                    "source_subnets": source_subnets,
                    "type": type,
                },
                app_edit_params.AppEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppEditResponse]], ResultWrapper[AppEditResponse]),
        )


class AsyncAppsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAppsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAppsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAppsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAppsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        type: str,
        hostnames: SequenceNotStr[str] | Omit = omit,
        ip_subnets: SequenceNotStr[str] | Omit = omit,
        source_subnets: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        """
        Creates a new App for an account

        Args:
          account_id: Identifier

          name: Display name for the app.

          type: Category of the app.

          hostnames: FQDNs to associate with traffic decisions.

          ip_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          source_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/magic/apps", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "hostnames": hostnames,
                    "ip_subnets": ip_subnets,
                    "source_subnets": source_subnets,
                },
                app_create_params.AppCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppCreateResponse]], ResultWrapper[AppCreateResponse]),
        )

    async def update(
        self,
        account_app_id: str,
        *,
        account_id: str,
        hostnames: SequenceNotStr[str] | Omit = omit,
        ip_subnets: SequenceNotStr[str] | Omit = omit,
        name: str | Omit = omit,
        source_subnets: SequenceNotStr[str] | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        """
        Updates an Account App

        Args:
          account_id: Identifier

          account_app_id: Identifier

          hostnames: FQDNs to associate with traffic decisions.

          ip_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          name: Display name for the app.

          source_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          type: Category of the app.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not account_app_id:
            raise ValueError(f"Expected a non-empty value for `account_app_id` but received {account_app_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/magic/apps/{account_app_id}",
                account_id=account_id,
                account_app_id=account_app_id,
            ),
            body=await async_maybe_transform(
                {
                    "hostnames": hostnames,
                    "ip_subnets": ip_subnets,
                    "name": name,
                    "source_subnets": source_subnets,
                    "type": type,
                },
                app_update_params.AppUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppUpdateResponse]], ResultWrapper[AppUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AppListResponse, AsyncSinglePage[AppListResponse]]:
        """
        Lists Apps associated with an account.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/apps", account_id=account_id),
            page=AsyncSinglePage[AppListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=cast(Any, AppListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        account_app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppDeleteResponse]:
        """
        Deletes specific Account App.

        Args:
          account_id: Identifier

          account_app_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not account_app_id:
            raise ValueError(f"Expected a non-empty value for `account_app_id` but received {account_app_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/apps/{account_app_id}",
                account_id=account_id,
                account_app_id=account_app_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppDeleteResponse]], ResultWrapper[AppDeleteResponse]),
        )

    async def edit(
        self,
        account_app_id: str,
        *,
        account_id: str,
        hostnames: SequenceNotStr[str] | Omit = omit,
        ip_subnets: SequenceNotStr[str] | Omit = omit,
        name: str | Omit = omit,
        source_subnets: SequenceNotStr[str] | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppEditResponse]:
        """
        Updates an Account App

        Args:
          account_id: Identifier

          account_app_id: Identifier

          hostnames: FQDNs to associate with traffic decisions.

          ip_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          name: Display name for the app.

          source_subnets: IPv4 CIDRs to associate with traffic decisions. (IPv6 CIDRs are currently
              unsupported)

          type: Category of the app.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not account_app_id:
            raise ValueError(f"Expected a non-empty value for `account_app_id` but received {account_app_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/apps/{account_app_id}",
                account_id=account_id,
                account_app_id=account_app_id,
            ),
            body=await async_maybe_transform(
                {
                    "hostnames": hostnames,
                    "ip_subnets": ip_subnets,
                    "name": name,
                    "source_subnets": source_subnets,
                    "type": type,
                },
                app_edit_params.AppEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppEditResponse]], ResultWrapper[AppEditResponse]),
        )


class AppsResourceWithRawResponse:
    def __init__(self, apps: AppsResource) -> None:
        self._apps = apps

        self.create = to_raw_response_wrapper(
            apps.create,
        )
        self.update = to_raw_response_wrapper(
            apps.update,
        )
        self.list = to_raw_response_wrapper(
            apps.list,
        )
        self.delete = to_raw_response_wrapper(
            apps.delete,
        )
        self.edit = to_raw_response_wrapper(
            apps.edit,
        )


class AsyncAppsResourceWithRawResponse:
    def __init__(self, apps: AsyncAppsResource) -> None:
        self._apps = apps

        self.create = async_to_raw_response_wrapper(
            apps.create,
        )
        self.update = async_to_raw_response_wrapper(
            apps.update,
        )
        self.list = async_to_raw_response_wrapper(
            apps.list,
        )
        self.delete = async_to_raw_response_wrapper(
            apps.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            apps.edit,
        )


class AppsResourceWithStreamingResponse:
    def __init__(self, apps: AppsResource) -> None:
        self._apps = apps

        self.create = to_streamed_response_wrapper(
            apps.create,
        )
        self.update = to_streamed_response_wrapper(
            apps.update,
        )
        self.list = to_streamed_response_wrapper(
            apps.list,
        )
        self.delete = to_streamed_response_wrapper(
            apps.delete,
        )
        self.edit = to_streamed_response_wrapper(
            apps.edit,
        )


class AsyncAppsResourceWithStreamingResponse:
    def __init__(self, apps: AsyncAppsResource) -> None:
        self._apps = apps

        self.create = async_to_streamed_response_wrapper(
            apps.create,
        )
        self.update = async_to_streamed_response_wrapper(
            apps.update,
        )
        self.list = async_to_streamed_response_wrapper(
            apps.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            apps.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            apps.edit,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["InstanceListParams"]


class InstanceListParams(TypedDict, total=False):
    account_id: Required[str]

    namespace: str
    """Filter by namespace."""

    order_by: Literal["created_at"]
    """Field to order results by."""

    order_by_direction: Literal["asc", "desc"]
    """Order direction."""

    page: int
    """Page number (1-indexed)."""

    per_page: int
    """Number of results per page."""

    search: str
    """Filter instances whose id contains this string (case-insensitive)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = ["InterconnectStatusResponse", "Pending", "Down", "Unhealthy", "Healthy"]


class Pending(BaseModel):
    state: Literal["Pending"]


class Down(BaseModel):
    state: Literal["Down"]

    reason: Optional[str] = None
    """Diagnostic information, if available"""


class Unhealthy(BaseModel):
    state: Literal["Unhealthy"]

    reason: Optional[str] = None
    """Diagnostic information, if available"""


class Healthy(BaseModel):
    state: Literal["Healthy"]


InterconnectStatusResponse: TypeAlias = Annotated[
    Union[Pending, Down, Unhealthy, Healthy], PropertyInfo(discriminator="state")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SubscriptionConfiguration", "StaleZoneConfiguration"]


class StaleZoneConfiguration(BaseModel):
    """
    A read-only field that shows which unauthorized settings are currently active on the zone. These settings typically result from upgrades or downgrades.
    """

    fight_mode: Optional[bool] = None
    """Indicates that the zone's Bot Fight Mode is turned on."""

    optimize_wordpress: Optional[bool] = None
    """Indicates that the zone's wordpress optimization for SBFM is turned on."""

    sbfm_definitely_automated: Optional[str] = None
    """
    Indicates that the zone's definitely automated requests are being blocked or
    challenged.
    """

    sbfm_likely_automated: Optional[str] = None
    """
    Indicates that the zone's likely automated requests are being blocked or
    challenged.
    """

    sbfm_static_resource_protection: Optional[str] = None
    """Indicates that the zone's static resource protection is turned on."""

    sbfm_verified_bots: Optional[str] = None
    """Indicates that the zone's verified bot requests are being blocked."""


class SubscriptionConfiguration(BaseModel):
    ai_bots_protection: Optional[Literal["block", "disabled", "only_on_ad_pages"]] = None
    """Enable rule to block AI Scrapers and Crawlers.

    Please note the value `only_on_ad_pages` is currently not available for
    Enterprise customers.
    """

    auto_update_model: Optional[bool] = None
    """
    Automatically update to the newest bot detection models created by Cloudflare as
    they are released.
    [Learn more.](https://developers.cloudflare.com/bots/reference/machine-learning-models#model-versions-and-release-notes)
    """

    bm_cookie_enabled: Optional[bool] = None
    """
    Indicates that the bot management cookie can be placed on end user devices
    accessing the site. Defaults to true
    """

    cf_robots_variant: Optional[Literal["off", "policy_only"]] = None
    """Specifies the Robots Access Control License variant to use."""

    content_bots_protection: Optional[Literal["block", "disabled"]] = None
    """Enable rule to block content bots.

    When enabled, blocks automated traffic with low bot scores, excluding safe
    verified bot categories. Exceptions should be managed via skip rules.
    """

    crawler_protection: Optional[Literal["enabled", "disabled"]] = None
    """Enable rule to punish AI Scrapers and Crawlers via a link maze."""

    enable_js: Optional[bool] = None
    """Use lightweight, invisible JavaScript detections to improve Bot Management.

    [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).
    """

    is_robots_txt_managed: Optional[bool] = None
    """Enable cloudflare managed robots.txt.

    If an existing robots.txt is detected, then managed robots.txt will be prepended
    to the existing robots.txt.
    """

    stale_zone_configuration: Optional[StaleZoneConfiguration] = None
    """
    A read-only field that shows which unauthorized settings are currently active on
    the zone. These settings typically result from upgrades or downgrades.
    """

    suppress_session_score: Optional[bool] = None
    """
    Whether to disable tracking the highest bot score for a session in the Bot
    Management cookie.
    """

    using_latest_model: Optional[bool] = None
    """
    A read-only field that indicates whether the zone currently is running the
    latest ML model.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.custom_nameservers import custom_nameserver_create_params
from ...types.custom_nameservers.custom_nameserver import CustomNameserver
from ...types.custom_nameservers.custom_nameserver_delete_response import CustomNameserverDeleteResponse

__all__ = ["CustomNameserversResource", "AsyncCustomNameserversResource"]


class CustomNameserversResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomNameserversResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomNameserversResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomNameserversResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomNameserversResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        ns_name: str,
        ns_set: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomNameserver]:
        """
        Add Account Custom Nameserver

        Args:
          account_id: Account identifier tag.

          ns_name: The FQDN of the name server.

          ns_set: The number of the set that this name server belongs to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/custom_ns", account_id=account_id),
            body=maybe_transform(
                {
                    "ns_name": ns_name,
                    "ns_set": ns_set,
                },
                custom_nameserver_create_params.CustomNameserverCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomNameserver]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomNameserver]], ResultWrapper[CustomNameserver]),
        )

    def delete(
        self,
        custom_ns_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CustomNameserverDeleteResponse]:
        """
        Delete Account Custom Nameserver

        Args:
          account_id: Account identifier tag.

          custom_ns_id: The FQDN of the name server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not custom_ns_id:
            raise ValueError(f"Expected a non-empty value for `custom_ns_id` but received {custom_ns_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/custom_ns/{custom_ns_id}", account_id=account_id, custom_ns_id=custom_ns_id
            ),
            page=SyncSinglePage[CustomNameserverDeleteResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
            method="delete",
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CustomNameserver]:
        """
        List an account's custom nameservers.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/custom_ns", account_id=account_id),
            page=SyncSinglePage[CustomNameserver],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CustomNameserver,
        )


class AsyncCustomNameserversResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomNameserversResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomNameserversResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomNameserversResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomNameserversResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        ns_name: str,
        ns_set: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomNameserver]:
        """
        Add Account Custom Nameserver

        Args:
          account_id: Account identifier tag.

          ns_name: The FQDN of the name server.

          ns_set: The number of the set that this name server belongs to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/custom_ns", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "ns_name": ns_name,
                    "ns_set": ns_set,
                },
                custom_nameserver_create_params.CustomNameserverCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomNameserver]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomNameserver]], ResultWrapper[CustomNameserver]),
        )

    def delete(
        self,
        custom_ns_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomNameserverDeleteResponse, AsyncSinglePage[CustomNameserverDeleteResponse]]:
        """
        Delete Account Custom Nameserver

        Args:
          account_id: Account identifier tag.

          custom_ns_id: The FQDN of the name server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not custom_ns_id:
            raise ValueError(f"Expected a non-empty value for `custom_ns_id` but received {custom_ns_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/custom_ns/{custom_ns_id}", account_id=account_id, custom_ns_id=custom_ns_id
            ),
            page=AsyncSinglePage[CustomNameserverDeleteResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
            method="delete",
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomNameserver, AsyncSinglePage[CustomNameserver]]:
        """
        List an account's custom nameservers.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/custom_ns", account_id=account_id),
            page=AsyncSinglePage[CustomNameserver],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CustomNameserver,
        )


class CustomNameserversResourceWithRawResponse:
    def __init__(self, custom_nameservers: CustomNameserversResource) -> None:
        self._custom_nameservers = custom_nameservers

        self.create = to_raw_response_wrapper(
            custom_nameservers.create,
        )
        self.delete = to_raw_response_wrapper(
            custom_nameservers.delete,
        )
        self.get = to_raw_response_wrapper(
            custom_nameservers.get,
        )


class AsyncCustomNameserversResourceWithRawResponse:
    def __init__(self, custom_nameservers: AsyncCustomNameserversResource) -> None:
        self._custom_nameservers = custom_nameservers

        self.create = async_to_raw_response_wrapper(
            custom_nameservers.create,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_nameservers.delete,
        )
        self.get = async_to_raw_response_wrapper(
            custom_nameservers.get,
        )


class CustomNameserversResourceWithStreamingResponse:
    def __init__(self, custom_nameservers: CustomNameserversResource) -> None:
        self._custom_nameservers = custom_nameservers

        self.create = to_streamed_response_wrapper(
            custom_nameservers.create,
        )
        self.delete = to_streamed_response_wrapper(
            custom_nameservers.delete,
        )
        self.get = to_streamed_response_wrapper(
            custom_nameservers.get,
        )


class AsyncCustomNameserversResourceWithStreamingResponse:
    def __init__(self, custom_nameservers: AsyncCustomNameserversResource) -> None:
        self._custom_nameservers = custom_nameservers

        self.create = async_to_streamed_response_wrapper(
            custom_nameservers.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_nameservers.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_nameservers.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .logging import Logging
from ..._models import BaseModel

__all__ = [
    "RewriteRule",
    "ActionParameters",
    "ActionParametersHeaders",
    "ActionParametersHeadersAddStaticHeader",
    "ActionParametersHeadersAddDynamicHeader",
    "ActionParametersHeadersSetStaticHeader",
    "ActionParametersHeadersSetDynamicHeader",
    "ActionParametersHeadersRemoveHeader",
    "ActionParametersURI",
    "ActionParametersURIURIPath",
    "ActionParametersURIURIPathPath",
    "ActionParametersURIURIQuery",
    "ActionParametersURIURIQueryQuery",
    "ExposedCredentialCheck",
    "Ratelimit",
]


class ActionParametersHeadersAddStaticHeader(BaseModel):
    """A header with a static value to add."""

    operation: Literal["add"]
    """The operation to perform on the header."""

    value: str
    """A static value for the header."""


class ActionParametersHeadersAddDynamicHeader(BaseModel):
    """A header with a dynamic value to add."""

    expression: str
    """An expression that evaluates to a value for the header."""

    operation: Literal["add"]
    """The operation to perform on the header."""


class ActionParametersHeadersSetStaticHeader(BaseModel):
    """A header with a static value to set."""

    operation: Literal["set"]
    """The operation to perform on the header."""

    value: str
    """A static value for the header."""


class ActionParametersHeadersSetDynamicHeader(BaseModel):
    """A header with a dynamic value to set."""

    expression: str
    """An expression that evaluates to a value for the header."""

    operation: Literal["set"]
    """The operation to perform on the header."""


class ActionParametersHeadersRemoveHeader(BaseModel):
    """A header to remove."""

    operation: Literal["remove"]
    """The operation to perform on the header."""


ActionParametersHeaders: TypeAlias = Union[
    ActionParametersHeadersAddStaticHeader,
    ActionParametersHeadersAddDynamicHeader,
    ActionParametersHeadersSetStaticHeader,
    ActionParametersHeadersSetDynamicHeader,
    ActionParametersHeadersRemoveHeader,
]


class ActionParametersURIURIPathPath(BaseModel):
    """A URI path rewrite."""

    expression: Optional[str] = None
    """An expression that evaluates to a value to rewrite the URI path to."""

    value: Optional[str] = None
    """A value to rewrite the URI path to."""


class ActionParametersURIURIPath(BaseModel):
    """A URI path rewrite."""

    path: ActionParametersURIURIPathPath
    """A URI path rewrite."""

    origin: Optional[bool] = None
    """Whether to propagate the rewritten URI to origin."""


class ActionParametersURIURIQueryQuery(BaseModel):
    """A URI query rewrite."""

    expression: Optional[str] = None
    """An expression that evaluates to a value to rewrite the URI query to."""

    value: Optional[str] = None
    """A value to rewrite the URI query to."""


class ActionParametersURIURIQuery(BaseModel):
    """A URI query rewrite."""

    query: ActionParametersURIURIQueryQuery
    """A URI query rewrite."""

    origin: Optional[bool] = None
    """Whether to propagate the rewritten URI to origin."""


ActionParametersURI: TypeAlias = Union[ActionParametersURIURIPath, ActionParametersURIURIQuery]


class ActionParameters(BaseModel):
    """The parameters configuring the rule's action."""

    headers: Optional[Dict[str, ActionParametersHeaders]] = None
    """A map of headers to rewrite."""

    uri: Optional[ActionParametersURI] = None
    """A URI path rewrite."""


class ExposedCredentialCheck(BaseModel):
    """Configuration for exposed credential checking."""

    password_expression: str
    """An expression that selects the password used in the credentials check."""

    username_expression: str
    """An expression that selects the user ID used in the credentials check."""


class Ratelimit(BaseModel):
    """An object configuring the rule's rate limit behavior."""

    characteristics: List[str]
    """
    Characteristics of the request on which the rate limit counter will be
    incremented.
    """

    period: int
    """Period in seconds over which the counter is being incremented."""

    counting_expression: Optional[str] = None
    """An expression that defines when the rate limit counter should be incremented.

    It defaults to the same as the rule's expression.
    """

    mitigation_timeout: Optional[int] = None
    """
    Period of time in seconds after which the action will be disabled following its
    first execution.
    """

    requests_per_period: Optional[int] = None
    """
    The threshold of requests per period after which the action will be executed for
    the first time.
    """

    requests_to_origin: Optional[bool] = None
    """Whether counting is only performed when an origin is reached."""

    score_per_period: Optional[int] = None
    """
    The score threshold per period for which the action will be executed the first
    time.
    """

    score_response_header_name: Optional[str] = None
    """
    A response header name provided by the origin, which contains the score to
    increment rate limit counter with.
    """


class RewriteRule(BaseModel):
    last_updated: datetime
    """The timestamp of when the rule was last modified."""

    version: str
    """The version of the rule."""

    id: Optional[str] = None
    """The unique ID of the rule."""

    action: Optional[Literal["rewrite"]] = None
    """The action to perform when the rule matches."""

    action_parameters: Optional[ActionParameters] = None
    """The parameters configuring the rule's action."""

    categories: Optional[List[str]] = None
    """The categories of the rule."""

    description: Optional[str] = None
    """An informative description of the rule."""

    enabled: Optional[bool] = None
    """Whether the rule should be executed."""

    exposed_credential_check: Optional[ExposedCredentialCheck] = None
    """Configuration for exposed credential checking."""

    expression: Optional[str] = None
    """The expression defining which traffic will match the rule."""

    logging: Optional[Logging] = None
    """An object configuring the rule's logging behavior."""

    ratelimit: Optional[Ratelimit] = None
    """An object configuring the rule's rate limit behavior."""

    ref: Optional[str] = None
    """The reference of the rule (the rule's ID by default)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.stream.live_inputs import output_create_params, output_update_params
from ....types.stream.live_inputs.output import Output

__all__ = ["OutputsResource", "AsyncOutputsResource"]


class OutputsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OutputsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OutputsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OutputsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OutputsResourceWithStreamingResponse(self)

    def create(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        stream_key: str,
        url: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Output]:
        """
        Creates a new output that can be used to simulcast or restream live video to
        other RTMP or SRT destinations. Outputs are always linked to a specific live
        input — one live input can have many outputs.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          stream_key: The streamKey used to authenticate against an output's target.

          url: The URL an output uses to restream.

          enabled: When enabled, live video streamed to the associated live input will be sent to
              the output URL. When disabled, live video will not be sent to the output URL,
              even when streaming to the associated live input. Use this to control precisely
              when you start and stop simulcasting to specific destinations like YouTube and
              Twitch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return self._post(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            body=maybe_transform(
                {
                    "stream_key": stream_key,
                    "url": url,
                    "enabled": enabled,
                },
                output_create_params.OutputCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Output]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Output]], ResultWrapper[Output]),
        )

    def update(
        self,
        output_identifier: str,
        *,
        account_id: str,
        live_input_identifier: str,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Output]:
        """
        Updates the state of an output.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          output_identifier: A unique identifier for the output.

          enabled: When enabled, live video streamed to the associated live input will be sent to
              the output URL. When disabled, live video will not be sent to the output URL,
              even when streaming to the associated live input. Use this to control precisely
              when you start and stop simulcasting to specific destinations like YouTube and
              Twitch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        if not output_identifier:
            raise ValueError(f"Expected a non-empty value for `output_identifier` but received {output_identifier!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs/{output_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
                output_identifier=output_identifier,
            ),
            body=maybe_transform({"enabled": enabled}, output_update_params.OutputUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Output]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Output]], ResultWrapper[Output]),
        )

    def list(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Output]:
        """
        Retrieves all outputs associated with a specified live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            page=SyncSinglePage[Output],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Output,
        )

    def delete(
        self,
        output_identifier: str,
        *,
        account_id: str,
        live_input_identifier: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes an output and removes it from the associated live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          output_identifier: A unique identifier for the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        if not output_identifier:
            raise ValueError(f"Expected a non-empty value for `output_identifier` but received {output_identifier!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs/{output_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
                output_identifier=output_identifier,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncOutputsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOutputsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOutputsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOutputsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOutputsResourceWithStreamingResponse(self)

    async def create(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        stream_key: str,
        url: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Output]:
        """
        Creates a new output that can be used to simulcast or restream live video to
        other RTMP or SRT destinations. Outputs are always linked to a specific live
        input — one live input can have many outputs.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          stream_key: The streamKey used to authenticate against an output's target.

          url: The URL an output uses to restream.

          enabled: When enabled, live video streamed to the associated live input will be sent to
              the output URL. When disabled, live video will not be sent to the output URL,
              even when streaming to the associated live input. Use this to control precisely
              when you start and stop simulcasting to specific destinations like YouTube and
              Twitch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return await self._post(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            body=await async_maybe_transform(
                {
                    "stream_key": stream_key,
                    "url": url,
                    "enabled": enabled,
                },
                output_create_params.OutputCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Output]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Output]], ResultWrapper[Output]),
        )

    async def update(
        self,
        output_identifier: str,
        *,
        account_id: str,
        live_input_identifier: str,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Output]:
        """
        Updates the state of an output.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          output_identifier: A unique identifier for the output.

          enabled: When enabled, live video streamed to the associated live input will be sent to
              the output URL. When disabled, live video will not be sent to the output URL,
              even when streaming to the associated live input. Use this to control precisely
              when you start and stop simulcasting to specific destinations like YouTube and
              Twitch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        if not output_identifier:
            raise ValueError(f"Expected a non-empty value for `output_identifier` but received {output_identifier!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs/{output_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
                output_identifier=output_identifier,
            ),
            body=await async_maybe_transform({"enabled": enabled}, output_update_params.OutputUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Output]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Output]], ResultWrapper[Output]),
        )

    def list(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Output, AsyncSinglePage[Output]]:
        """
        Retrieves all outputs associated with a specified live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            page=AsyncSinglePage[Output],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Output,
        )

    async def delete(
        self,
        output_identifier: str,
        *,
        account_id: str,
        live_input_identifier: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes an output and removes it from the associated live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          output_identifier: A unique identifier for the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        if not output_identifier:
            raise ValueError(f"Expected a non-empty value for `output_identifier` but received {output_identifier!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}/outputs/{output_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
                output_identifier=output_identifier,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class OutputsResourceWithRawResponse:
    def __init__(self, outputs: OutputsResource) -> None:
        self._outputs = outputs

        self.create = to_raw_response_wrapper(
            outputs.create,
        )
        self.update = to_raw_response_wrapper(
            outputs.update,
        )
        self.list = to_raw_response_wrapper(
            outputs.list,
        )
        self.delete = to_raw_response_wrapper(
            outputs.delete,
        )


class AsyncOutputsResourceWithRawResponse:
    def __init__(self, outputs: AsyncOutputsResource) -> None:
        self._outputs = outputs

        self.create = async_to_raw_response_wrapper(
            outputs.create,
        )
        self.update = async_to_raw_response_wrapper(
            outputs.update,
        )
        self.list = async_to_raw_response_wrapper(
            outputs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            outputs.delete,
        )


class OutputsResourceWithStreamingResponse:
    def __init__(self, outputs: OutputsResource) -> None:
        self._outputs = outputs

        self.create = to_streamed_response_wrapper(
            outputs.create,
        )
        self.update = to_streamed_response_wrapper(
            outputs.update,
        )
        self.list = to_streamed_response_wrapper(
            outputs.list,
        )
        self.delete = to_streamed_response_wrapper(
            outputs.delete,
        )


class AsyncOutputsResourceWithStreamingResponse:
    def __init__(self, outputs: AsyncOutputsResource) -> None:
        self._outputs = outputs

        self.create = async_to_streamed_response_wrapper(
            outputs.create,
        )
        self.update = async_to_streamed_response_wrapper(
            outputs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            outputs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            outputs.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ....workers import script as _script
from ....._models import BaseModel

__all__ = ["Script"]


class Script(BaseModel):
    """Details about a worker uploaded to a Workers for Platforms namespace."""

    created_on: Optional[datetime] = None
    """When the script was created."""

    dispatch_namespace: Optional[str] = None
    """Name of the Workers for Platforms dispatch namespace."""

    modified_on: Optional[datetime] = None
    """When the script was last modified."""

    script: Optional[_script.Script] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "InterconnectCreateResponse",
    "NscInterconnectPhysicalBody",
    "NscInterconnectPhysicalBodyFacility",
    "NscInterconnectGcpPartnerBody",
]


class NscInterconnectPhysicalBodyFacility(BaseModel):
    address: List[str]

    name: str


class NscInterconnectPhysicalBody(BaseModel):
    account: str

    facility: NscInterconnectPhysicalBodyFacility

    name: str

    site: str
    """A Cloudflare site name."""

    slot_id: str

    speed: str

    type: str

    owner: Optional[str] = None


class NscInterconnectGcpPartnerBody(BaseModel):
    account: str

    name: str

    region: str

    type: str

    owner: Optional[str] = None

    speed: Optional[Literal["50M", "100M", "200M", "300M", "400M", "500M", "1G", "2G", "5G", "10G", "20G", "50G"]] = (
        None
    )
    """Bandwidth structure as visible through the customer-facing API."""


InterconnectCreateResponse: TypeAlias = Union[NscInterconnectPhysicalBody, NscInterconnectGcpPartnerBody]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ScriptSearchParams"]


class ScriptSearchParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    id: str
    """Worker ID (also called tag) to search for. Only exact matches are returned."""

    name: str
    """Worker name to search for. Both exact and partial matches are returned."""

    order_by: Literal["created_on", "modified_on", "name"]
    """Property to sort results by. Results are sorted in ascending order."""

    page: int
    """Current page."""

    per_page: int
    """Items per page."""
